/* $Id: fsu_mkfifo.c,v 1.2 2008/08/01 10:41:58 stacktic Exp $ */

/*
 * Copyright (c) 2008 Arnaud Ysmal.  All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <err.h>
#include <getopt.h>
#include <stdio.h>
#include <stdlib.h>

#include <rump/ukfs.h>

#include "fsu_utils.h"

int
fsu_mkfifo_main(struct ukfs *fs, int argc, char *argv[])
{
	int cur_arg, res, rv;
	void *set;
	mode_t mode;

	mode = 0666;
	while ((rv = getopt(argc, argv, "m:")) != -1) {
		switch(rv) {
		case 'm':
			set = setmode(optarg);
			if (set == NULL) {
				warn("Cannot set file mode `%s'", optarg);
				return -1;
			}
		
			mode = getmode(set, 0666);
			free(set);
			break;
		case '?':
		default:
			fprintf(stderr, "usage: mkfifo [-m mode] fifoname\n");
			return -1;
		}
	}
	argc -= optind;
	argv += optind;

	if (argc < 1) {
		fprintf(stderr, "usage: mkfifo [-m mode] fifoname\n");
		return -1;
	}

	res = 0;
	for (cur_arg = 0; cur_arg < argc; ++cur_arg)
		res |= fsu_mkfifo(fs, argv[cur_arg], mode);

	return res;
}

int
fsu_mkfifo(struct ukfs *fs, char *fifoname, mode_t mode)
{
	int rv;

	rv = ukfs_mkfifo(fs, fifoname, mode);
	if (rv == -1) {
		warn("%s", fifoname);
		return -1;
	}
	return 0;
}
