/* $Id: main.c,v 1.4 2008/08/13 17:20:17 stacktic Exp $ */

/*
 * Copyright (c) 2008 Arnaud Ysmal.  All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <err.h>
#include <stdio.h>
#include <stdlib.h>

#include <rump/ukfs.h>

#include "fsu_utils.h"
#include "fsu_mount.h"
#include "fsu_touch.h"

static void usage(void);

int
main(int argc, char *argv[])
{
	int cur_arg, flags, rv;
	struct ukfs *fs;
	char *rtime, *ttime;
	
	setprogname(argv[0]);

	fs = fsu_mount(&argc, &argv, NULL, NULL);
	if (fs == NULL) {
		warn("fsu_mount");
		usage();
		/* NOTREACHED */
	}

	flags = fsu_touch_parse_arg(&argc, &argv, &rtime, &ttime);
	if (flags == -1 || argc < 1)
		usage();

	for (rv = 0, cur_arg = 0; cur_arg < argc; ++cur_arg)
		rv |= fsu_touch(fs, argv[cur_arg], rtime, ttime, flags);

	ukfs_release(fs, 0);
	return rv;
}

static void
usage(void)
{

	fprintf(stderr, "usage: "
   "%s [MNT_OPTIONS] [fstype] fsdevice [-achm] [-r file] [-t time] file ...\n",
		getprogname());
	exit(EXIT_FAILURE);
}
