/*	$Id: fsu_write.c,v 1.4 2008/08/13 17:20:18 stacktic Exp $	*/

/*
 * Copyright (c) 2008 Arnaud Ysmal.  All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>

#include <err.h>
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <locale.h>
#include <time.h>
#include <tzfile.h>
#include <unistd.h>

#include <rump/ukfs.h>

#include "fsu_utils.h"
#include "fsu_mount.h"
#include "fsu_write.h"

int
fsu_write_main(struct ukfs *fs, int argc, char *argv[])
{
	int flags, rv;
	char *comname;

	comname = argv[0];

	flags = fsu_write_parse_arg(&argc, &argv);

	if (flags == -1 || argc != 1) {
		fprintf(stderr,
			"usage: %s [-a] file\n",
			comname);
		return -1;
	}

	rv = fsu_write(fs, STDIN_FILENO, argv[0], flags);

	return rv == -1 ? 1 : 0;
}

int
fsu_write(struct ukfs *fs, int fd, const char *fname, int flags)
{
	struct stat sb;
	off_t off;
	int rd, wr;
	uint8_t buf[8192];
	mode_t mask;

	off = 0;

	rd = ukfs_stat(fs, fname, &sb);

	if (!(flags & FSU_WRITE_APPEND) && rd == 0) {
		ukfs_remove(fs, fname);
		rd = ukfs_create(fs, fname, sb.st_mode);
		if (rd == -1) {
			warn("%s", fname);
			return -1;
		}
	}

	umask((mask = umask(0)));

	if (rd == -1) {
		rd = ukfs_create(fs, fname, 0777 & ~mask);
		if (rd == -1) {
			warn("%s", fname);
			return -1;
		}
	} else if (flags & FSU_WRITE_APPEND)
		off = sb.st_size;

	do {
		rd = read(fd, buf, sizeof(buf));
		if (rd == -1) {
			warn("read");
			return -1;
		}
		wr = ukfs_write(fs, fname, off, buf, rd);
		if (wr == -1 || wr != rd) {
			warn("%s", fname);
			return -1;
		}
		off += rd;
	} while (rd > 0 && errno != ENOSPC);

	return off;
}

int
fsu_write_parse_arg(int *argc, char ***argv)
{
	int ch, flags;

	flags = 0;
	while ((ch = getopt(*argc, *argv, "a")) != -1) {
		switch(ch) {
		case 'a':
			flags |= FSU_WRITE_APPEND;
			break;

		case '?':
		default:
			return -1;
		}
		}
	*argc -= optind;
	*argv += optind;
	
	return flags;
}
