/* $Id */

/*
 * Copyright (c) 2008 Arnaud Ysmal.  All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <rump/ukfs.h>

#include "fsu_file.h"

char
fsu_fgetc(FSU_FILE *file)
{
	int rv;

	if (file->fd_fpos == file->fd_last) {
		if (file->fd_eof)
			return EOF;
		
		rv = ukfs_read(file->fd_ukfs, file->fd_name, file->fd_fpos, 
			       file->fd_buf, sizeof(file->fd_buf));
		if (rv == -1) {
			file->fd_err = errno;
			return EOF;
		}
		file->fd_last += rv;
		if (rv < sizeof(file->fd_buf))
			file->fd_eof = true;
		file->fd_bpos = 0;
	}
	++file->fd_fpos;
	return file->fd_buf[file->fd_bpos++];
}

FSU_FILE
*fsu_fopen(struct ukfs *fs, const char *fname)
{
	FSU_FILE *file;

	file = malloc(sizeof(FSU_FILE));
	if (file == NULL)
		return NULL;
	file->fd_name = strdup(fname);
	if (file->fd_name == NULL) {
		free(file);
		return NULL;
	}
	file->fd_ukfs = fs;
	file->fd_err = file->fd_fpos = file->fd_bpos = file->fd_last = 0;
	file->fd_eof = false;
	return file;
}

void
fsu_fclose(FSU_FILE *file)
{

	free(file->fd_name);
	free(file);
}

void
fsu_rewind(FSU_FILE *file)
{
	
	file->fd_fpos = file->fd_bpos = file->fd_last = 0;
	return;
}

bool
fsu_feof(FSU_FILE *file)
{

	return file->fd_eof;
}
	
void
fsu_clearerr(FSU_FILE *file)
{

	file->fd_err = 0;
}

int
fsu_ferror(FSU_FILE *file)
{

	return file->fd_err;
}
